@@ -1,5 +1,5 @@
 // Paraselene generated this source file.
-package #1.base;
+package #1.#x;
 
 import paraselene.*;
 import paraselene.supervisor.*;
