@@ -6,10 +6,16 @@
  */
 package paraselene;
 
+import paraselene.tag.*;
 import java.io.*;
 import java.net.*;
 
 
+class DummyTag extends Tag {
+	DummyTag(){ super( "dummy", false ); }
+	public void setModify() {}
+}
+
 /**
  * HTML構造。スレッドセーフです。
  */
@@ -43,16 +49,31 @@ public abstract class HTMLPart implements Serializable {
 	};
 	private static final long serialVersionUID = 2L;
 
+	private static final DummyTag	null_tag = new DummyTag();
 	/**
-	 * 所属先ページ。
+	 * フレームワークが使用します。
 	 */
-	protected Page	embed = null;
-
+	public static Tag getDummyTag() {
+		return null_tag;
+	}
 	/**
-	 * 所属ページの設定。
+	 * 所属先タグ。
 	 */
-	public void setPage( Page p ) {
-		embed = p;
+	private Tag	embed_tag = null_tag;
+	/**
+	 * 所属先タグの設定。
+	 * @param t 所属先。
+	 */
+	public void setAssignedTag( Tag t ) {
+		if ( t == null )	t = null_tag;
+		embed_tag = t;
+	}
+	/**
+	 * 所属先タグの取得。
+	 * @return 所属先。
+	 */
+	public Tag getAssignedTag() {
+		return embed_tag;
 	}
 
 	protected HTMLPart() {}
