@@ -70,7 +70,7 @@ public class NumberValue extends PlainText implements AttributeValuable, CSSValu
 		/**
 		 * センチメートル。
 		 */
-		CENTIMETERM( "cm" ),
+		CENTIMETER( "cm" ),
 		/**
 		 * ミリメートル。
 		 */
@@ -451,7 +451,7 @@ public class NumberValue extends PlainText implements AttributeValuable, CSSValu
 	 * @return this。
 	 */
 	public synchronized NumberValue add( double p ) {
-		val += p;
+		setNumber( val + p );
 		return this;
 	}
 
@@ -481,7 +481,7 @@ public class NumberValue extends PlainText implements AttributeValuable, CSSValu
 	 * @return this。
 	 */
 	public synchronized NumberValue subtract( double p ) {
-		val -= p;
+		setNumber( val - p );
 		return this;
 	}
 
@@ -511,7 +511,7 @@ public class NumberValue extends PlainText implements AttributeValuable, CSSValu
 	 * @return this。
 	 */
 	public synchronized NumberValue multiply( double p ) {
-		val *= p;
+		setNumber( val * p );
 		return this;
 	}
 
@@ -537,12 +537,13 @@ public class NumberValue extends PlainText implements AttributeValuable, CSSValu
 
 	/**
 	 * 除算。this / p を設定します。thisとリターン値は同じものです。
+	 * pが0なら値は変化しません。
 	 * @param p 除算値。
 	 * @return this。
 	 */
 	public synchronized NumberValue divide( double p ) {
 		if ( p == 0 )	return this;
-		val /= p;
+		setNumber( val / p );
 		return this;
 	}
 
