@@ -27,6 +27,9 @@ public abstract class Page implements Serializable {
 	public class PageException extends Exception {
 		private Throwable ex = null;
 		private Object	obj = null;
+		private Page	page;
+		private RequestParameter	request;
+		private boolean	input_call;
 		/**
 		 * コンストラクタ。
 		 * 別の例外をラップします。
@@ -39,6 +42,9 @@ public abstract class Page implements Serializable {
 				e = ((PageException)e).ex;
 			}
 			ex = e;
+			page = SandBox.getCurrentPage();
+			request = SandBox.getCurrentRequestParameter();
+			input_call = SandBox.isCurrentInput();
 		}
 
 		/**
@@ -49,6 +55,9 @@ public abstract class Page implements Serializable {
 		public PageException( String mes ) {
 			super( mes );
 			ex = null;
+			page = SandBox.getCurrentPage();
+			request = SandBox.getCurrentRequestParameter();
+			input_call = SandBox.isCurrentInput();
 		}
 
 		/**
@@ -94,6 +103,30 @@ public abstract class Page implements Serializable {
 		public Object getWithObject() {
 			return obj;
 		}
+
+		/**
+		 * ページの取得。
+		 * @return 例外発生元のページインスタンス。
+		 */
+		public Page getPage() {
+			return page;
+		}
+		
+		/**
+		 * リクエストパラメータの取得。
+		 * @return 例外発生時のリクエストパラメータ。
+		 */
+		public RequestParameter getRequestParameter() {
+			return request;
+		}
+
+		/**
+		 * Input処理中であるか。
+		 * @return true:input、false:input以外 output や firstOutput。
+		 */
+		public boolean isInput() {
+			return input_call;
+		}
 	}
 
 	/**
@@ -125,6 +158,22 @@ public abstract class Page implements Serializable {
 	 */
 	public abstract AjaxSupport getAjaxSupport();
 
+	private volatile int histry_key = 0;
+	/**
+	 * 画面遷移履歴キーの設定。
+	 * @param key 画面遷移履歴キー。
+	 */
+	public void setHistoryKey( int key ) {
+		histry_key = key;
+	}
+	/**
+	 * 画面遷移履歴キーの取得。
+	 * @return 画面遷移履歴キー。
+	 */
+	public int getHistoryKey() {
+		return histry_key;
+	}
+
 	private static final int ERROR_NO = 403;
 
 	private HashMap<String,Tag>	tag_1_dic = new HashMap<String,Tag>();
@@ -698,13 +747,27 @@ public abstract class Page implements Serializable {
 		clearDic( name );
 	}
 
+	private void clearAssignedPage( Tag tag ) {
+		tag.setAssignedPage( null );
+		Tag[]	next = tag.getTagArray();
+		if ( next == null )	return;
+		for ( int i = 0; i < next.length; i++ ) {
+			clearAssignedPage( next[i] );
+		}
+	}
+
 	/**
 	 * HTMLタグの登録。
 	 * @param tag HTMLタグ。
 	 */
 	public synchronized void setMainTag( Tag tag ) {
+		if ( contents != null ) {
+			clearAssignedPage( contents );
+		}
 		contents = tag;
-		nameTag( tag );
+		if ( tag != null ) {
+			nameTag( tag );
+		}
 	}
 
 	/**
@@ -827,16 +890,16 @@ public abstract class Page implements Serializable {
 
 	/**
 	 * 更新日時を返す。<br>
-	 * nullを返すと更新日なしとなり、ブラウザキャッシュを禁止します。<br>
+	 * nullを返すと更新日なしとなり、ブラウザへのキャッシュを抑制します。<br>
 	 * null以外を返すと、ブラウザへ更新日時を返しキャッシュを許可します。<br>
 	 * null以外を返した際にブラウザがIf-Modified-Since要求している場合、
 	 * この戻り値と比較し、更新されていなければ304(not modified)をブラウザへ返し、
 	 * 実データは送信されません。<br>
 	 * nullを返すと、常に実データをブラウザへ送信します。
-	 * @return 常にnullを返し、キャッシュを禁止します。
+	 * @return 常にnullを返し、キャッシュを抑制します。
 	 * 必要に応じてオーバーライドして下さい。<br>
-	 * ブログであれば記事の投稿日、画像であれば画像ファイルのタイムスタンプを
-	 * 返すとブラウザキャッシュが有効に働き通信量を減らせます。
+	 * ブログであれば記事の投稿日、画像であれば画像ファイルのタイムスタンプ、
+	 * 等を返すとブラウザキャッシュが有効に働き通信量を減らせます。
 	 */
 	public HTTPDate getLastModified() {
 		return null;
