@@ -450,7 +450,7 @@ public class URIValue extends PlainText implements AttributeValuable, CSSValuabl
 				ret = URIValue.encode( super.toString( StringMode.PLAIN ), getCharset() );
 			}
 			else {
-				ret = Supervisor.makeWithSessionURI( super.toString( StringMode.PLAIN ), getCharset() );
+				ret = Supervisor.makeWithSessionURI( super.toString( StringMode.PLAIN ), getCharset(), getAssignedTag() );
 			}
 		}
 		catch( Exception e ){
