@@ -437,13 +437,22 @@ public class RequestParameter {
 
 	/**
 	 * 履歴の取得。
+	 * @param key 画面遷移履歴キー。
 	 * @return 履歴。セッションが発生していない場合はnull。
 	 */
-	public History getHistory() {
+	public History getHistory( int key ) {
 		HttpSession	session = getSession();
 		if ( session == null )	return null;
 		SessionData	data = (SessionData)session.getAttribute( Supervisor.SESSION_DATA );
-		return data.hist;
+		return data.hist.get( key );
+	}
+
+	/**
+	 * 履歴の取得。
+	 * @return 履歴。セッションが発生していない場合はnull。
+	 */
+	public History getHistory() {
+		return getHistory( SandBox.getCurrentPage().getHistoryKey() );
 	}
 
 	/**
