@@ -39,9 +39,9 @@ class RedirectData implements Serializable {
 class SessionData implements Serializable {
 	private static final long serialVersionUID = 2L;
 	TransactionSequencer seq = new TransactionSequencer();
-	History hist = new History();
+	HistorySet hist = new HistorySet();
 	private HashMap<PageID, RedirectData>	pending = new HashMap<PageID, RedirectData>();
-	private int client;
+	private volatile int client;
 
 	private int getClient( HttpServletRequest req ) {
 		String[]	data = new String[] {
