@@ -9,6 +9,7 @@ package paraselene.supervisor;
 
 import paraselene.*;
 import paraselene.ajax.*;
+import paraselene.tag.*;
 import java.io.*;
 import java.net.*;
 import java.util.*;
@@ -72,7 +73,7 @@ public abstract class Supervisor extends HttpServlet {
 		super();
 	}
 
-	private static String resolvePage( DataHolder holder, String p ) {
+	private static String resolvePage( DataHolder holder, String p, int from_key, int to_key ) {
 		try {
 			URI	u = new URI( p );
 			PageID	to = null;
@@ -88,7 +89,7 @@ public abstract class Supervisor extends HttpServlet {
 					TransactionSequencer.makeURI(
 						( page.isCheckRepeatSameRequest() )?
 							holder.data.seq: null,
-						holder.myself, to
+						holder.myself, to, from_key, to_key, !holder.search_engine
 					)
 				);
 				String	query = u.getQuery();
@@ -129,20 +130,42 @@ public abstract class Supervisor extends HttpServlet {
 		}
 	}
 
+	private static String getTarget( Tag tag ) {
+		if ( tag == null )	return null;
+		String	name = tag.getName();
+		String	ret = null;
+		if ( "a".equals( name ) || "form".equals( name ) ) {
+			ret = tag.getAttributeToString( "target" );
+		}
+		else if ( "frame".equals( name ) || "iframe".equals( name ) ) {
+			ret = tag.getAttributeToString( "name" );
+			if ( ret == null ) {
+				ret = tag.getAttributeToString( "id" );
+			}
+		}
+		return ret;
+	}
+
 	/**
 	 * URIへのセッション情報付加。
 	 * @param uri URI。
 	 * @param enc エンコード文字コード。
+	 * @param tag タグ。
 	 * @return セッション情報が追加されたURI。
 	 */
-	public static String makeWithSessionURI( String uri, String enc ) 
+	public static String makeWithSessionURI( String uri, String enc, Tag tag ) 
 	throws UnsupportedEncodingException {
 		DataHolder	holder = null;
 		synchronized( data_map_th ) {
 			holder = data_map_th.get( Thread.currentThread() );
 		}
 		if ( holder == null )	return URIValue.encode( uri, enc );
-		if ( holder.search_engine )	return URIValue.encode( uri, enc );
+		int from_key, to_key;
+		if ( tag != null ) {
+			from_key = tag.getAssignedPage().getHistoryKey();
+			to_key = holder.data.hist.toInt( getTarget( tag ), from_key );
+		}
+		else	from_key = to_key = HistorySet.ROOT;
 		String[]	part = uri.split( "/" );
 		boolean	flag = false;
 		String	head = null;
@@ -157,7 +180,7 @@ public abstract class Supervisor extends HttpServlet {
 			}
 			switch( def_no ) {
 			case 0:
-				part[i] = resolvePage( holder, part[i] );
+				part[i] = resolvePage( holder, part[i], from_key, to_key );
 				break;
 			case 1:
 			case 2:
@@ -173,8 +196,23 @@ public abstract class Supervisor extends HttpServlet {
 				buf = buf.append( part[i] );
 			}
 			uri = buf.toString();
+			uri = holder.response.encodeURL( URIValue.encode( uri, enc ) );
+		}
+		else {
+			try {
+				uri = URIValue.encode( uri, enc );
+				URI	test = new URI( uri );
+				if ( !test.isAbsolute() ) {
+					uri = holder.response.encodeURL( uri );
+				}
+				else if ( test.getHost().equals( holder.req.getServerName() ) ) {
+					uri = holder.response.encodeURL( uri );
+				}
+			}
+			catch( Exception e ) {
+				Option.debug( e );
+			}
 		}
-		uri = holder.response.encodeURL( URIValue.encode( uri, enc ) );
 		if ( head != null ) {
 			StringBuilder	buf = new StringBuilder( head );
 			buf = buf.append( "/" );
@@ -272,7 +310,7 @@ public abstract class Supervisor extends HttpServlet {
 			Page	last_page = null, next_page = null;
 
 			if ( (path_data.to = fact.getPage( uri )) == null ) {
-				path_data = TransactionSequencer.parsePath( data.seq, uri, fact );
+				path_data = TransactionSequencer.parsePath( data.seq, uri, fact, data.hist );
 				if ( TransactionSequencer.isMustSession( uri ) ) {
 					if ( holder.search_engine ) {
 						new Forward( 403 ).Action( this, res );
@@ -322,7 +360,7 @@ public abstract class Supervisor extends HttpServlet {
 					return;
 				}
 				if ( last_page.isAllowHistoryAdd() ) {
-					Page	sub = data.hist.getPage( last_page.getID() );
+					Page	sub = data.hist.get( (path_data.from_histry != null)? path_data.from_histry: HistorySet.ROOT ).getPage( last_page.getID() );
 					if ( sub != null ) {
 						fact.returnPage( last_page );
 						last_page = sub;
@@ -375,8 +413,25 @@ public abstract class Supervisor extends HttpServlet {
 				}
 			}
 			if ( fw == null ) {
+				int	from_key = HistorySet.ROOT;
+				int	to_key = HistorySet.ROOT;
+				if ( path_data.from_histry != null )	from_key = path_data.from_histry;
+				if ( path_data.to_histry != null )	to_key = path_data.to_histry;
 				if ( next_page != null ) {
-					fw = new Forward( next_page.getID(), true, false );
+					if ( from_key != to_key ) {
+						fw = new Forward( next_page.getID(), true, false );
+					}
+					else {
+						if ( last_page.getID() == next_page.getID() ) {
+							fw = new Feedback( next_page.getID() );
+						}
+						else if ( Closure.isClosure( last_page, next_page, req ) ) {
+							fw = new Closure( next_page.getID() );
+						}
+						else {
+							fw = new Forward( next_page.getID(), true, false );
+						}
+					}
 					fact.returnPage( next_page );
 				}
 				last_page.setRequestParameter( req );
@@ -386,6 +441,9 @@ public abstract class Supervisor extends HttpServlet {
 				last_page.setRequestParameter( null );
 				holder.history_use = fw.history_f;
 				session_f = fw.session_clear_f;
+				if ( fw.getPageID() == Closure.CLOSE ) {
+					fw = new Feedback( last_page.getID() );
+				}
 				next_page = fw.getPage( last_page );
 				if ( next_page == null ) {
 					fw.Action( this, res );
@@ -393,7 +451,11 @@ public abstract class Supervisor extends HttpServlet {
 					return;
 				}
 			}
-			Page[]	out_ret = output_call( last_page, next_page );
+			int	out_hist_key = HistorySet.ROOT;
+			if ( path_data.to_histry != null ) {
+				out_hist_key = path_data.to_histry;
+			}
+			Page[]	out_ret = output_call( out_hist_key, last_page, next_page );
 			if ( out_ret == null ) {
 				System.out.println( "output_call false" );
 				fw = new Forward( 500 );
@@ -485,7 +547,7 @@ public abstract class Supervisor extends HttpServlet {
 				try {
 					if ( date.compareTo( new HTTPDate( mod[0] ) ) <= 0 ) {
 						res.sendError( 304 );
-						data.hist.add( next_page );
+						data.hist.get( next_page.getHistoryKey() ).add( next_page );
 						return;
 					}
 				}
@@ -505,7 +567,7 @@ public abstract class Supervisor extends HttpServlet {
 				}
 			}
 			if ( gzip( param, holder.req, res, out_ret[1] ) ) {
-				data.hist.add( next_page );
+				data.hist.get( next_page.getHistoryKey() ).add( next_page );
 				return;
 			}
 			if ( out_ret[1] instanceof Downloadable ) {
@@ -537,7 +599,7 @@ public abstract class Supervisor extends HttpServlet {
 				w.flush();
 			}
 		}
-		data.hist.add( next_page );
+		data.hist.get( next_page.getHistoryKey() ).add( next_page );
 	}
 
 	private boolean gzip( RequestParameter param, HttpServletRequest req, HttpServletResponse res, Page page )  throws Exception {
@@ -603,7 +665,8 @@ public abstract class Supervisor extends HttpServlet {
 		return true;
 	}
 
-	private Page[] output_call( Page last_page, Page next_page ) throws Throwable {
+	private Page[] output_call( int next_key, Page last_page, Page next_page ) throws Throwable {
+		next_page.setHistoryKey( next_key );
 		PageID	last_id = null;
 		if ( last_page instanceof NullPage ) {
 			last_page = null;
@@ -627,7 +690,7 @@ public abstract class Supervisor extends HttpServlet {
 				next_page = last_page;
 			}
 			else if ( next_page.isAllowHistoryAdd() ) {
-				Page	sub = data.hist.getPage( next_page.getID() );
+				Page	sub = data.hist.get( next_key ).getPage( next_page.getID() );
 				if ( sub != null ) {
 					getPageFactory().returnPage( next_page );
 					next_page = sub;
@@ -691,7 +754,7 @@ public abstract class Supervisor extends HttpServlet {
 		holder.response = res;
 		holder.data = data;
 
-		PathData	p = TransactionSequencer.parsePath( null, request.getRequestURI() , getPageFactory() );
+		PathData	p = TransactionSequencer.parsePath( null, request.getRequestURI() , getPageFactory(), null );
 		if ( p != null ) {
 			if ( p.from != null ) {
 				request.setCharacterEncoding( p.from.getCharset() );
