@@ -288,11 +288,23 @@ public class Tag extends HTMLPart {
 	/**
 	 * 属性の取得。無ければnullを返す。
 	 * @param at_name 属性名。
+	 * @return 属性。
 	 */
 	public synchronized Attribute getAttribute( String at_name ) {
 		return attr.get( at_name.toLowerCase( Locale.ENGLISH ) );
 	}
 
+	/**
+	 * 属性値を文字列にして取得。無ければnullを返す。
+	 * @param at_name 属性名。
+	 * @return 属性値の文字列。
+	 */
+	public String getAttributeToString( String at_name ) {
+		Attribute	attr = getAttribute( at_name );
+		if ( attr == null )	return null;
+		return attr.getString();
+	}
+
 	/**
 	 * HTML文書の追加。name属性値が衝突すると、それを変更します。
 	 * 末尾に追加されます。
