@@ -50,9 +50,7 @@ public class SingleTextBox extends Input {
 		if ( !hidden_f )	return super.getFirstTag();
 
 		boolean	xml = false;
-		if ( embed != null ) {
-			xml = embed.isXML();
-		}
+		xml = getAssignedTag().getAssignedPage().isXML();
 		StringBuilder	buf = new StringBuilder( "<" );
 		buf = buf.append( getName() );
 		for ( String k : attr.keySet() ) {
